﻿using System;
using System.Collections.Generic;
using System.Linq;
using Orchard.UI.Resources;

namespace Kendo.Web
{
    public class KendoResourceManifest : IResourceManifestProvider
    {
        public static string MainScript = "kendo";
        private string VersionPath(string version, string file) 
        {
            return string.Format("{0}/{1}", version, file);
        }

        /// <summary>
        /// Defines the scripts.
        /// </summary>
        /// <param name="manifest">The manifest.</param>
        /// <param name="key">The key.</param>
        /// <param name="currentVersion">The current version.</param>
        /// <param name="previousVersions">The previous versions.</param>
        public void DefineScripts(ResourceManifest manifest, string key, string currentVersion, params string[] previousVersions)
        {
            var versions = //new List<string>(previousVersions) { currentVersion };
                previousVersions.Concat(new[] { currentVersion });
            foreach (var ver in versions)
            {
                string webKey = string.Format("{0}.{1}", key, "web");
                string webData = string.Format("{0}.{1}", key, "webdata");
                string dataVizKey = string.Format("{0}.{1}", key, "data");
                string mobileKey = string.Format("{0}.{1}", key, "mobile");
                string all = string.Format("{0}.{1}", key, "all");

                manifest.DefineScript(webKey)
                    .SetDependencies("jQuery")
                    .SetUrl(VersionPath(ver, "kendo.web.min.js"))
                    .SetDependencies("jQuery")
                    .SetAttribute("kendo-version", ver)
                    //.SetCdn("http://cdn.kendostatic.com/2012.2.913/js/kendo.web.min.js")
                    //.SetDependencies(webData)
                    .SetVersion(ver);

                manifest.DefineScript(dataVizKey)
                    .SetDependencies("jQuery")
                    .SetUrl(VersionPath(ver, "kendo.dataviz.min.js"))
                    .SetDependencies("jQuery")
                    //.SetDependencies(webData)
                    .SetVersion(ver);

                manifest.DefineScript(mobileKey).SetUrl(VersionPath(ver, "kendo.mobile.min.js"))
                    .SetDependencies("jQuery")
                    .SetVersion(ver);

                manifest.DefineScript(webData)
                    .SetUrl(VersionPath(ver, "kendo.webdata.min.js")).SetAttribute("kendo-version", ver)
                    .SetDependencies("jQuery")
                    .SetVersion(ver);
            }

            //found issue: http://orchard.codeplex.com/workitem/19255
            //manifest.DefineScript("kendo.web")
            //    .SetDependencies("jQuery")
            //    .SetUrl("2012.2.810/kendo.web.min.js");

            manifest.DefineScript("kendo.datasource")
                .SetDependencies("kendo.web")
                .SetUrl("custom/datasource.js");
                
        }

        /// <summary>
        /// Gets the key format.
        /// </summary>
        /// <value>The key format.</value>
        public static string KeyFormat
        {
            get
            {
                return "{0}.{1}";
            }
        }

        /// <summary>
        /// Defines the styles.
        /// </summary>
        /// <param name="manifest">The manifest.</param>
        /// <param name="key">The key.</param>
        /// <param name="currentVersion">The current version.</param>
        /// <param name="previousVersions">The previous versions.</param>
        public void DefineStyles(ResourceManifest manifest, string key, string currentVersion, params string[] previousVersions) 
        {
            var versions = new List<string>(previousVersions) { currentVersion };
            //versions.AddRange(previousVersions);

            Func<string, string, string> format = (a,b) => {
                return string.Format(KeyFormat, a, b);
            };

            string commonDependency = format(key, "common");

            foreach (var version in versions) 
            { 
                manifest.DefineStyle(format(key, "common"))
                    .SetUrl(VersionPath(version, "kendo.common.min.css"))
                    .SetVersion(version);

                manifest.DefineStyle(format(key, "default"))
                    .SetUrl(VersionPath(version, "kendo.default.min.css"))
                    .SetDependencies(commonDependency)
                    .SetVersion(version);

                manifest.DefineStyle(format(key, "black"))
                    .SetUrl(VersionPath(version, "kendo.black.min.css"))
                    .SetDependencies(commonDependency)
                    .SetVersion(version);

                manifest.DefineStyle(format(key, "blueopal"))
                    .SetUrl(VersionPath(version, "kendo.blueopal.min.css"))
                    .SetDependencies(commonDependency)
                    .SetVersion(version);

                manifest.DefineStyle(format(key, "metro"))
                    .SetUrl(VersionPath(version, "kendo.metro.min.css"))
                    .SetDependencies(commonDependency)
                    .SetVersion(version);

                manifest.DefineStyle(format(key, "silver"))
                    .SetUrl(VersionPath(version, "kendo.silver.min.css"))
                    .SetDependencies(commonDependency)
                    .SetVersion(version);

            }
        }

        public void DefineCultureSet(ResourceManifest manifest, string key, string currentVersion, params string[] previousVersions)
        {
            manifest.DefineScript("kendo.culture.en-gb")
                .SetUrl("culture/kendo.culture.en-gb.js")
                .SetDependencies(MainScript);

        }

        public void BuildManifests(ResourceManifestBuilder builder)
        {
            var manifest = builder.Add();

            var mainScript = MainScript;

            string currentVerison = "2012.3.1114";
            //string currentVerison = "2012.2.710";
            string[] versions = new[] 
            {
                "2012.2.710", 
                "2012.2.621", 
                //"2012.1.327" removed to save some space for module upload
            };
            //string[] versions = new string[0] {  };


            this.DefineScripts(manifest, mainScript, currentVerison, versions);
            this.DefineStyles(manifest, mainScript, currentVerison, versions);
            this.DefineCultureSet(manifest, mainScript, currentVerison, versions);
        }

    }
}
